@@ -35,9 +35,10 @@ int main(int argc, char *argv[])
 
     catch (RelionError XE)
     {
-        prm.usage();
+    	if (prm.verb > 0)
+    		prm.usage();
         std::cout << XE;
-        exit(1);
+        MPI_Abort(MPI_COMM_WORLD, EXIT_FAILURE);
     }
 
     return 0;
