@@ -37,7 +37,7 @@ int main(int argc, char *argv[])
     {
         prm.usage();
         std::cout << XE;
-        exit(1);
+        MPI_Abort(MPI_COMM_WORLD, EXIT_FAILURE);
     }
 
     return 0;
