@@ -32,8 +32,8 @@ class reconstruct_parameters
    	FileName fn_out, fn_sel, fn_img, fn_sym, fn_sub, fn_fsc, fn_debug;
 	MetaDataTable DF;
 	int r_max, r_min_nn, blob_order, padding_factor, ref_dim, interpolator, iter, nr_threads, debug_ori_size, debug_size, ctf_dim;
-	double blob_radius, blob_alpha, angular_error, shift_error, angpix, maxres, beamtilt_x, beamtilt_y;
-	bool do_ctf, ctf_phase_flipped, only_flip_phases, intact_ctf_first_peak, do_fom_weighting, do_beamtilt;
+	DOUBLE blob_radius, blob_alpha, angular_error, shift_error, angpix, maxres, beamtilt_x, beamtilt_y;
+	bool do_ctf, ctf_phase_flipped, only_flip_phases, intact_ctf_first_peak, do_fom_weighting, do_3d_rot, do_reconstruct_ctf, do_beamtilt;
 	// I/O Parser
 	IOParser parser;
 
@@ -85,6 +85,11 @@ class reconstruct_parameters
     	shift_error = textToFloat(parser.getOption("--shift_error", "Apply random deviations with this standard deviation (in pixels) to each of the 2 translations", "0."));
     	do_fom_weighting = parser.checkOption("--fom_weighting", "Weight particles according to their figure-of-merit (_rlnParticleFigureOfMerit)");
     	fn_fsc = parser.getOption("--fsc", "FSC-curve for regularized reconstruction", "");
+    	do_3d_rot = parser.checkOption("--3d_rot", "Perform 3D rotations instead of backprojections from 2D images");
+    	ctf_dim  = textToInteger(parser.getOption("--reconstruct_ctf", "Perform a 3D reconstruction from 2D CTF-images, with the given size in pixels", "-1"));
+    	do_reconstruct_ctf = (ctf_dim > 0);
+    	if (do_reconstruct_ctf)
+    		do_ctf = false;
 
     	// Hidden
        	r_min_nn = textToInteger(getParameter(argc, argv, "--r_min_nn", "10"));
@@ -107,13 +112,14 @@ class reconstruct_parameters
 	void reconstruct()
 	{
 
+		int data_dim = (do_3d_rot) ? 3 : 2;
 		if (fn_debug != "")
 		{
-			BackProjector backprojector(debug_ori_size, 3, fn_sym, interpolator, padding_factor, r_min_nn, blob_order, blob_radius, blob_alpha);
+			BackProjector backprojector(debug_ori_size, 3, fn_sym, interpolator, padding_factor, r_min_nn, blob_order, blob_radius, blob_alpha, data_dim);
 			backprojector.initialiseDataAndWeight(debug_size);
 			backprojector.data.printShape();
 			backprojector.weight.printShape();
-			Image<double> It;
+			Image<DOUBLE> It;
 			It.read(fn_debug+"_data_real.mrc");
 			It().setXmippOrigin();
 			It().xinit=0;
@@ -138,7 +144,7 @@ class reconstruct_parameters
 				A3D_ELEM(backprojector.weight, k, i, j) = A3D_ELEM(It(), k, i, j);
 			}
 
-			MultidimArray<double> dummy;
+			MultidimArray<DOUBLE> dummy;
 			backprojector.reconstruct(It(), iter, false, 1., dummy, dummy, dummy, dummy, 1., false, true, nr_threads, -1);
 	    	It.write(fn_out);
 	    	std::cerr<<" Done writing map in "<<fn_out<<std::endl;
@@ -148,13 +154,13 @@ class reconstruct_parameters
 		else
 		{
 
-		double rot, tilt, psi, fom;
-		Matrix2D<double> A3D;
+		DOUBLE rot, tilt, psi, fom;
+		Matrix2D<DOUBLE> A3D;
 		MultidimArray<Complex > Faux, F2D, Fsub;
-		MultidimArray<double> Fweight, Fctf, dummy;
-		Image<double> vol, img, sub;
+		MultidimArray<DOUBLE> Fweight, Fctf, dummy;
+		Image<DOUBLE> vol, img, sub;
 		FourierTransformer transformer;
-		Matrix1D< double > trans(2);
+		Matrix1D< DOUBLE > trans(2);
 		Projector proj;
 		int mysize;
 //#define DEBUG_WW
@@ -168,7 +174,7 @@ class reconstruct_parameters
 		BackProjector backprojectort(mysize, ref_dim, fn_sym, interpolator, padding_factor, r_min_nn, blob_order, blob_radius, blob_alpha);
 		backprojectort.initZeros(2 * r_max);
 
-		Image<double> Imagn, Iphas, Iw, tvol;
+		Image<DOUBLE> Imagn, Iphas, Iw, tvol;
 		Imagn.read("FEW_it24_rank2_data_magn.spi");
 		Iphas.read("FEW_it24_rank2_data_phas.spi");
 		Iw.read("FEW_it24_rank2_weight.spi");
@@ -176,7 +182,7 @@ class reconstruct_parameters
         Iw().xinit=0;
 
 		// Write out invw
-		Image<double> oo;
+		Image<DOUBLE> oo;
 		oo=Iw;
 		FOR_ALL_DIRECT_ELEMENTS_IN_MULTIDIMARRAY(Iw())
 		{
@@ -190,8 +196,8 @@ class reconstruct_parameters
 		backprojectort.weight.printShape();
 		FOR_ALL_DIRECT_ELEMENTS_IN_MULTIDIMARRAY(Imagn())
 		{
-			double realval = sin(DIRECT_MULTIDIM_ELEM(Iphas(), n)) * DIRECT_MULTIDIM_ELEM(Imagn(), n);
-			double imagval = cos(DIRECT_MULTIDIM_ELEM(Iphas(), n)) * DIRECT_MULTIDIM_ELEM(Imagn(), n);
+			DOUBLE realval = sin(DIRECT_MULTIDIM_ELEM(Iphas(), n)) * DIRECT_MULTIDIM_ELEM(Imagn(), n);
+			DOUBLE imagval = cos(DIRECT_MULTIDIM_ELEM(Iphas(), n)) * DIRECT_MULTIDIM_ELEM(Imagn(), n);
 			DIRECT_MULTIDIM_ELEM(backprojectort.data, n) = (Complex)(realval, imagval);
 		}
 		backprojectort.weight = Iw();
@@ -204,21 +210,31 @@ class reconstruct_parameters
 
 
    		// Get dimension of the images
-		(DF).firstObject();
-		DF.getValue(EMDL_IMAGE_NAME, fn_img);
-		img.read(fn_img);
-		mysize=(int)XSIZE(img());
+   		if (do_reconstruct_ctf)
+   		{
+   			mysize = ctf_dim;
+   			img().resize(ctf_dim, ctf_dim);
+   			img().setXmippOrigin();
+   		}
+   		else
+   		{
+			(DF).firstObject();
+			DF.getValue(EMDL_IMAGE_NAME, fn_img);
+			img.read(fn_img);
+			mysize=(int)XSIZE(img());
+   		}
+
 
    		if (DF.containsLabel(EMDL_CTF_MAGNIFICATION) && DF.containsLabel(EMDL_CTF_DETECTOR_PIXEL_SIZE))
     	{
-    		double mag, dstep;
+    		DOUBLE mag, dstep;
    			DF.getValue(EMDL_CTF_MAGNIFICATION, mag);
    			DF.getValue(EMDL_CTF_DETECTOR_PIXEL_SIZE, dstep);
    			angpix = 10000. * dstep / mag;
    			std::cout << " + Using pixel size calculated from magnification and detector pixel size in the input STAR file: " << angpix << std::endl;
     	}
 
-   		BackProjector backprojector(mysize, ref_dim, fn_sym, interpolator, padding_factor, r_min_nn, blob_order, blob_radius, blob_alpha);
+   		BackProjector backprojector(mysize, ref_dim, fn_sym, interpolator, padding_factor, r_min_nn, blob_order, blob_radius, blob_alpha, data_dim);
 
    		if (maxres < 0.)
    			r_max = -1;
@@ -245,9 +261,12 @@ class reconstruct_parameters
    		FOR_ALL_OBJECTS_IN_METADATA_TABLE(DF)
    		{
 
-   			DF.getValue(EMDL_IMAGE_NAME, fn_img);
-   			img.read(fn_img);
-   			img().setXmippOrigin();
+   			if (!do_reconstruct_ctf)
+   			{
+   				DF.getValue(EMDL_IMAGE_NAME, fn_img);
+   				img.read(fn_img);
+   				img().setXmippOrigin();
+   			}
 
 			// Rotations
 			if (ref_dim==2)
@@ -280,6 +299,13 @@ class reconstruct_parameters
    				XX(trans) += rnd_gaus(0., shift_error);
    				YY(trans) += rnd_gaus(0., shift_error);
    			}
+  			if (do_3d_rot)
+   			{
+   				trans.resize(3);
+   				DF.getValue( EMDL_ORIENT_ORIGIN_Z, ZZ(trans));
+   	  			if (shift_error > 0.)
+   	   				ZZ(trans) += rnd_gaus(0., shift_error);
+   			}
 
    			if (do_fom_weighting)
    				DF.getValue( EMDL_PARTICLE_FOM, fom);
@@ -290,12 +316,17 @@ class reconstruct_parameters
    			CenterFFT(img(), true);
    			transformer.FourierTransform(img(), F2D);
    			if (ABS(XX(trans)) > 0. || ABS(YY(trans)) > 0.)
-   				shiftImageInFourierTransform(F2D, F2D, XSIZE(img()), trans );
+   			{
+   				if (do_3d_rot)
+   					shiftImageInFourierTransform(F2D, F2D, XSIZE(img()), XX(trans), YY(trans), ZZ(trans));
+   				else
+   					shiftImageInFourierTransform(F2D, F2D, XSIZE(img()), XX(trans), YY(trans));
+   			}
 
 			Fctf.resize(F2D);
 			Fctf.initConstant(1.);
 			// Apply CTF if necessary
-			if (do_ctf)
+			if (do_ctf || do_reconstruct_ctf)
 			{
 				CTF ctf;
 				ctf.read(DF, DF);
@@ -336,7 +367,15 @@ class reconstruct_parameters
 			else
 			{
 				// "Normal" reconstruction, multiply X by CTF, and W by CTF^2
-				if (do_ctf)
+				if (do_reconstruct_ctf)
+				{
+					FOR_ALL_DIRECT_ELEMENTS_IN_MULTIDIMARRAY(F2D)
+					{
+						DIRECT_MULTIDIM_ELEM(F2D, n)  = DIRECT_MULTIDIM_ELEM(Fctf, n);
+						DIRECT_MULTIDIM_ELEM(Fctf, n) = 1.;
+					}
+				}
+				else if (do_ctf)
 				{
 					FOR_ALL_DIRECT_ELEMENTS_IN_MULTIDIMARRAY(F2D)
 					{
@@ -369,7 +408,7 @@ class reconstruct_parameters
 						//std::cerr << " exp_R_mic= " << exp_R_mic << std::endl;
 						std::cerr << " rot= " << rot << " tilt= " << tilt << " psi= " << psi << " xoff= "<< XX(trans)<< " yoff= "<<YY(trans)<<std::endl;
 						//std::cerr << "mic_id= "<<mic_id<<" mymodel.sigma2_noise[mic_id]= " << mymodel.sigma2_noise[mic_id] << std::endl;
-						Image<double> It;
+						Image<DOUBLE> It;
 						It()=Fctf;
 						It.write("reconstruct_Fctf.spi");
 						It().resize(mysize, mysize);
@@ -395,7 +434,7 @@ class reconstruct_parameters
 
    		bool do_map = false;
    		bool do_use_fsc = false;
-   		MultidimArray<double> fsc;
+   		MultidimArray<DOUBLE> fsc;
    		fsc.resize(mysize/2+1);
    		if (fn_fsc != "")
    		{
@@ -406,7 +445,7 @@ class reconstruct_parameters
    			FOR_ALL_OBJECTS_IN_METADATA_TABLE(MDfsc)
    			{
    				int idx;
-   				double val;
+   				DOUBLE val;
    				MDfsc.getValue(EMDL_SPECTRAL_IDX, idx);
    				MDfsc.getValue(EMDL_MLMODEL_FSC_HALVES_REF, val);
    				fsc(idx) =  val;
@@ -415,6 +454,31 @@ class reconstruct_parameters
    		std::cerr << "Starting the reconstruction ..." << std::endl;
    		backprojector.reconstruct(vol(), iter, do_map, 1., dummy, dummy, dummy, fsc, 1., do_use_fsc, true, nr_threads, -1);
 
+   		if (do_reconstruct_ctf)
+   		{
+
+   			F2D.clear();
+   			transformer.FourierTransform(vol(), F2D);
+
+   			// CenterOriginFFT: Set the center of the FFT in the FFTW origin
+			Matrix1D<DOUBLE> shift(3);
+			XX(shift)=-(DOUBLE)(int)(ctf_dim / 2);
+			YY(shift)=-(DOUBLE)(int)(ctf_dim / 2);
+			ZZ(shift)=-(DOUBLE)(int)(ctf_dim / 2);
+			shiftImageInFourierTransform(F2D, F2D, (DOUBLE)ctf_dim, XX(shift), YY(shift), ZZ(shift));
+			vol().setXmippOrigin();
+   			vol().initZeros();
+   			FOR_ALL_ELEMENTS_IN_FFTW_TRANSFORM(F2D)
+   			{
+   				// Take care of kp==dim/2, as XmippOrigin lies just right off center of image...
+   				if ( kp > FINISHINGZ(vol()) || ip > FINISHINGY(vol()) || jp > FINISHINGX(vol()))
+   					continue;
+   				A3D_ELEM(vol(), kp, ip, jp)    = FFTW_ELEM(F2D, kp, ip, jp).real;
+   				A3D_ELEM(vol(), -kp, -ip, -jp) = FFTW_ELEM(F2D, kp, ip, jp).real;
+   			}
+   			vol() *= (DOUBLE)ctf_dim;
+   		}
+
    		vol.write(fn_out);
     	std::cerr<<" Done writing map in "<<fn_out<<std::endl;
 
