@@ -43,15 +43,12 @@ int main(int argc, char **argv)
         {
         	optimiser.usage();
         }
-    	std::cerr << XE;
-
+    	std::cout << XE;
         MPI_Abort(MPI_COMM_WORLD, EXIT_FAILURE);
 
     }
 
-    MPI_Finalize();
-
-    exit(0);
+    return 0;
 
 }
 
