@@ -151,7 +151,7 @@ public:
      * An empty image is created.
      *
      * @code
-     * Image<double> I;
+     * Image<DOUBLE> I;
      * @endcode
      */
     Image()
@@ -411,13 +411,13 @@ public:
             }
         case Double:
                 {
-                    if (typeid(T) == typeid(double))
+                    if (typeid(T) == typeid(DOUBLE))
                 {
                     memcpy(ptrDest, page, pageSize*sizeof(T));
                     }
                     else
                     {
-                        double * ptr = (double *) page;
+                        DOUBLE * ptr = (DOUBLE *) page;
                         for(int i=0; i<pageSize; i++)
                             ptrDest[i]=(T) ptr[i];
                     }
@@ -456,15 +456,15 @@ public:
             }
         case Double:
                 {
-                    if (typeid(T) == typeid(double))
+                    if (typeid(T) == typeid(DOUBLE))
                 {
                     memcpy(page, srcPtr, pageSize*sizeof(T));
                     }
                     else
                     {
-                        double * ptr = (double *) page;
+                        DOUBLE * ptr = (DOUBLE *) page;
                         for(int i=0; i<pageSize; i++)
-                            ptr[i] = (double)srcPtr[i];
+                            ptr[i] = (DOUBLE)srcPtr[i];
                     }
                 break;
             }
@@ -580,7 +580,7 @@ public:
             }
         case Double:
             {
-                if (typeid(T) == typeid(double))
+                if (typeid(T) == typeid(DOUBLE))
                     return 1;
                 else
                     return 0;
@@ -888,9 +888,9 @@ public:
     * std::cout << "sampling= " << samplingRateX() << std::endl;
     * @endcode
     */
-    double samplingRateX(const long int n = 0) const
+    DOUBLE samplingRateX(const long int n = 0) const
     {
-        double dummy = 1.;
+        DOUBLE dummy = 1.;
         MDMainHeader.getValue(EMDL_IMAGE_SAMPLINGRATE_X, dummy);
         return dummy;
     }
@@ -901,9 +901,9 @@ public:
     * std::cout << "sampling= " << samplingRateY() << std::endl;
     * @endcode
     */
-    double samplingRateY(const long int n = 0) const
+    DOUBLE samplingRateY(const long int n = 0) const
     {
-        double dummy = 1.;
+        DOUBLE dummy = 1.;
         MDMainHeader.getValue(EMDL_IMAGE_SAMPLINGRATE_Y, dummy);
         return dummy;
     }
@@ -920,7 +920,7 @@ public:
      */
     void setStatisticsInHeader()
     {
-    	double avg,stddev,minval,maxval;
+    	DOUBLE avg,stddev,minval,maxval;
     	data.computeStats(avg, stddev, minval, maxval);
     	MDMainHeader.setValue(EMDL_IMAGE_STATS_AVG, avg);
     	MDMainHeader.setValue(EMDL_IMAGE_STATS_STDDEV, stddev);
@@ -928,7 +928,7 @@ public:
     	MDMainHeader.setValue(EMDL_IMAGE_STATS_MAX, maxval);
     }
 
-    void setSamplingRateInHeader(double rate_x, double rate_y = -1., double rate_z = -1.)
+    void setSamplingRateInHeader(DOUBLE rate_x, DOUBLE rate_y = -1., DOUBLE rate_z = -1.)
     {
     	MDMainHeader.setValue(EMDL_IMAGE_SAMPLINGRATE_X, rate_x);
     	if (rate_y < 0.)
@@ -1303,22 +1303,22 @@ private:
 
 // Some image-specific operations
 
-// For image normalization
-void normalise(Image<double> &I, int bg_radius, double white_dust_stddev, double black_dust_stddev);
-
-void calculateBackgroundAvgStddev(Image<double> &I, double &avg, double &stddev, int bg_radius);
+// For image normalisation
+void normalise(Image<DOUBLE> &I, int bg_radius, DOUBLE white_dust_stddev, DOUBLE black_dust_stddev, bool do_ramp);
+void calculateBackgroundAvgStddev(Image<DOUBLE> &I, DOUBLE &avg, DOUBLE &stddev, int bg_radius);
+void subtractBackgroundRamp(Image<DOUBLE> &I, int bg_radius);
 
 // For dust removal
-void removeDust(Image<double> &I, bool is_white, double thresh, double avg, double stddev);
+void removeDust(Image<DOUBLE> &I, bool is_white, DOUBLE thresh, DOUBLE avg, DOUBLE stddev);
 
 // for contrast inversion
-void invert_contrast(Image<double> &I);
+void invert_contrast(Image<DOUBLE> &I);
 
 // for image re-scaling
-void rescale(Image<double> &I, int mysize);
+void rescale(Image<DOUBLE> &I, int mysize);
 
 // for image re-windowing
-void rewindow(Image<double> &I, int mysize);
+void rewindow(Image<DOUBLE> &I, int mysize);
 
 
 
