@@ -76,7 +76,7 @@ enum EMDLabel
     EMDL_CTF_CA, ///< Chromatic aberration
     EMDL_CTF_DETECTOR_PIXEL_SIZE, ///< Pixel size for detector as used in CTF-determination
     EMDL_CTF_ENERGY_LOSS, ///< Energy loss
-    EMDL_CTF_FOM, ///< ctffind3 FOM (CC) for quality of CTF-fit
+    EMDL_CTF_FOM, ///< ctffind FOM (CC) for quality of CTF-fit
     EMDL_CTF_IMAGE, ///< name of an image describing the CTF model
     EMDL_CTF_LENS_STABILITY, ///< Lens stability
     EMDL_CTF_MAGNIFICATION, ///< Magnification used for CTF-determination
@@ -88,6 +88,7 @@ enum EMDLabel
     EMDL_CTF_VALUE, ///< CTF value
 
     EMDL_IMAGE_NAME,
+    EMDL_IMAGE_ORI_NAME,
     EMDL_IMAGE_RECONSTRUCT_NAME,
     EMDL_IMAGE_ID,
     EMDL_IMAGE_ENABLED,
@@ -267,6 +268,7 @@ enum EMDLabel
 
     EMDL_POSTPROCESS_BFACTOR,
     EMDL_POSTPROCESS_FINAL_RESOLUTION,
+    EMDL_POSTPROCESS_FSC_GENERAL,
     EMDL_POSTPROCESS_FSC_TRUE,
     EMDL_POSTPROCESS_FSC_MASKED,
     EMDL_POSTPROCESS_FSC_UNMASKED,
@@ -411,11 +413,12 @@ private:
         EMDL::addLabel(EMDL_CTF_VALUE, EMDL_DOUBLE, "rlnCtfValue", "Value of the Contrast Transfer Function");
 
     	EMDL::addLabel(EMDL_IMAGE_NAME, EMDL_STRING, "rlnImageName", "Name of an image");
+    	EMDL::addLabel(EMDL_IMAGE_ORI_NAME, EMDL_STRING, "rlnImageOriginalName", "Original name of an image");
     	EMDL::addLabel(EMDL_IMAGE_RECONSTRUCT_NAME, EMDL_STRING, "rlnReconstructImageName", "Name of an image to be used for reconstruction only");
     	EMDL::addLabel(EMDL_IMAGE_ID, EMDL_LONG, "rlnImageId", "ID (i.e. a unique number) of an image");
     	EMDL::addLabel(EMDL_IMAGE_ENABLED, EMDL_BOOL, "rlnEnabled", "Not used in RELION, only included for backward compatibility with XMIPP selfiles");
-        EMDL::addLabel(EMDL_IMAGE_DATATYPE, EMDL_INT, "rlnDataType", "Type of data stored in an image (e.g. int, double etc)");
-        EMDL::addLabel(EMDL_IMAGE_DIMENSIONALITY, EMDL_INT, "rlnDataDimensionality", "Dimensionality of data stored in an image (i.e. 2 or 3)");
+        EMDL::addLabel(EMDL_IMAGE_DATATYPE, EMDL_INT, "rlnDataType", "Type of data stored in an image (e.g. int, DOUBLE etc)");
+        EMDL::addLabel(EMDL_IMAGE_DIMENSIONALITY, EMDL_INT, "rlnImageDimensionality", "Dimensionality of data stored in an image (i.e. 2 or 3)");
         EMDL::addLabel(EMDL_IMAGE_BEAMTILT_X, EMDL_DOUBLE, "rlnBeamTiltX", "Beam tilt in the X-direction (in mrad)");
         EMDL::addLabel(EMDL_IMAGE_BEAMTILT_Y, EMDL_DOUBLE, "rlnBeamTiltY", "Beam tilt in the Y-direction (in mrad)");
         EMDL::addLabel(EMDL_IMAGE_BEAMTILT_GROUP, EMDL_STRING, "rlnBeamTiltGroupName", "Name of a group (of images) with assumedly identical beam-tilts");
@@ -590,6 +593,7 @@ private:
 
         EMDL::addLabel(EMDL_POSTPROCESS_FINAL_RESOLUTION, EMDL_DOUBLE, "rlnFinalResolution", "Final estimated resolution after postprocessing (in Angstroms)");
         EMDL::addLabel(EMDL_POSTPROCESS_BFACTOR, EMDL_DOUBLE, "rlnBfactorUsedForSharpening", "Applied B-factor in the sharpening of the map");
+        EMDL::addLabel(EMDL_POSTPROCESS_FSC_GENERAL, EMDL_DOUBLE, "rlnFourierShellCorrelation", "FSC value (of unspecified type, e.g. masked or unmasked)");
         EMDL::addLabel(EMDL_POSTPROCESS_FSC_TRUE, EMDL_DOUBLE, "rlnFourierShellCorrelationCorrected", "Final FSC value: i.e. after correction based on masking of randomized-phases maps");
         EMDL::addLabel(EMDL_POSTPROCESS_FSC_MASKED, EMDL_DOUBLE, "rlnFourierShellCorrelationMaskedMaps", "FSC value after masking of the original maps");
         EMDL::addLabel(EMDL_POSTPROCESS_FSC_UNMASKED, EMDL_DOUBLE, "rlnFourierShellCorrelationUnmaskedMaps", "FSC value before masking of the original maps");
