@@ -44,6 +44,72 @@
 
 #include "src/metadata_table.h"
 
+// Label for sorting
+static EMDLabel sortlabel;
+bool compareString(MetaDataContainer *lh, MetaDataContainer *rh)
+{
+	std::string lhstr, rhstr;
+	lh->getValue(sortlabel, lhstr);
+	rh->getValue(sortlabel, rhstr);
+	return lhstr < rhstr;
+}
+bool compareStringNoAt(MetaDataContainer *lh, MetaDataContainer *rh)
+{
+	std::string lhstr, rhstr;
+	lh->getValue(sortlabel, lhstr);
+	rh->getValue(sortlabel, rhstr);
+	lhstr = lhstr.substr(lhstr.find("@")+1);
+	rhstr = rhstr.substr(rhstr.find("@")+1);
+	return lhstr < rhstr;
+}
+bool compareDouble(MetaDataContainer *lh, MetaDataContainer *rh)
+{
+	DOUBLE lhstr, rhstr;
+	lh->getValue(sortlabel, lhstr);
+	rh->getValue(sortlabel, rhstr);
+	return lhstr < rhstr;
+}
+bool compareInteger(MetaDataContainer *lh, MetaDataContainer *rh)
+{
+	int lhstr, rhstr;
+	lh->getValue(sortlabel, lhstr);
+	rh->getValue(sortlabel, rhstr);
+	return lhstr < rhstr;
+}
+bool compareLong(MetaDataContainer *lh, MetaDataContainer *rh)
+{
+	long lhstr, rhstr;
+	lh->getValue(sortlabel, lhstr);
+	rh->getValue(sortlabel, rhstr);
+	return lhstr < rhstr;
+}
+
+void MetaDataTable::newSort(const EMDLabel label, bool do_reverse, bool do_sort_after_at)
+{
+
+	sortlabel = label;
+	if (EMDL::isString(label))
+	{
+		if (do_sort_after_at)
+			std::stable_sort(objects.begin(), objects.end(), compareStringNoAt);
+		else
+			std::stable_sort(objects.begin(), objects.end(), compareString);
+	}
+	else if (EMDL::isDouble(label))
+		std::stable_sort(objects.begin(), objects.end(), compareDouble);
+	else if (EMDL::isInt(label))
+		std::stable_sort(objects.begin(), objects.end(), compareInteger);
+	else if (EMDL::isLong(label))
+		std::stable_sort(objects.begin(), objects.end(), compareLong);
+	else
+		REPORT_ERROR("Cannot sort this label: " + EMDL::label2Str(label));
+
+	if (do_reverse)
+		std::reverse(objects.begin(), objects.end());
+
+}
+
+
 MetaDataTable::MetaDataTable()
 {
     clear();
@@ -156,49 +222,23 @@ std::string MetaDataTable::getName() const
     return name;
 }
 
-bool MetaDataTable::setValue(const std::string &name, const std::string &value,
+bool MetaDataTable::setValueFromString(const EMDLabel &label, const std::string &value,
                         long int objectID)
 {
-    EMDLabel label = EMDL::str2Label(name);
 
-    if (!isEmpty() && EMDL::isValidLabel(label))
-    {
-        long int auxID = (objectID == -1) ? current_objectID : objectID;
+	if (objectID == -1)
+		objectID = current_objectID;
 
-        if (auxID >= objects.size())
-        	REPORT_ERROR("MetaDataTable::setValue: auxID >= objects.size()");
+	if (objectID >= objects.size())
+		REPORT_ERROR("MetaDataTable::setValueFromString: objectID >= objects.size()");
 
-        MetaDataContainer * aux = objects[auxID];
+	objects[objectID]->addValueFromString(label, value);
 
-        // Check whether label is correct (belongs to the enum in the metadata_container header
-        // and whether it is present in the activeLabels vector. If not, add it to all the other
-        // objects with default values
-        std::vector<EMDLabel>::iterator location;
-        location = std::find(activeLabels.begin(), activeLabels.end(), label);
+	return true;
 
-        if (location == activeLabels.end())
-        {
-            activeLabels.push_back(label);
-
-            // Add this label to the rest of the objects in this class
-            for (long int idx = 0; idx < objects.size(); idx++)
-            {
-                if (objects[idx] != aux)
-                	objects[idx]->addDefaultValue(label);
-            }
-        }
-
-        aux->addValue(name, value);
-
-        return true;
-    }
-    else
-    {
-    	// in case of empty objects or invalid label
-        return false;
-    }
 }
 
+
 bool MetaDataTable::containsLabel(const EMDLabel label) const
 {
     return vectorContainsLabel(activeLabels, label);
@@ -241,7 +281,7 @@ long int MetaDataTable::addObject(MetaDataContainer * data, long int objectID)
 
     if (objectID == -1)
     {
-        result = objects.size();
+    	result = objects.size();
         objects.resize(result+1);
     }
     else
@@ -269,11 +309,13 @@ long int MetaDataTable::addObject(MetaDataContainer * data, long int objectID)
     // Set default values for the existing labels
     if (data == NULL)
     {
-		std::vector<EMDLabel>::iterator It;
+    	/* Sjors 21nov2014: I don't see why this would be necessary...
+    	std::vector<EMDLabel>::iterator It;
 		for (It = activeLabels.begin(); It != activeLabels.end(); It++)
 		{
 			(objects[result])->addDefaultValue(*It);
 		}
+		*/
     }
     else
     {
@@ -538,7 +580,7 @@ void MetaDataTable::readStarLoop(std::ifstream& in, std::vector<EMDLabel> *desir
 				labelPosition++;
 				continue;
 			}
-			setValue(EMDL::label2Str(activeLabels[labelPosition - counterIgnored]), value);
+			setValueFromString(activeLabels[labelPosition - counterIgnored], value);
 			labelPosition++;
 		}
 
@@ -580,7 +622,7 @@ bool MetaDataTable::readStarList(std::ifstream& in, std::vector<EMDLabel> *desir
     		 if (label != EMDL_UNDEFINED)
 			 {
 				 activeLabels.push_back(label);
-				 setValue(EMDL::label2Str(label), value, objectID);
+				 setValueFromString(label, value, objectID);
 			 }
     	 }
     	 // Check whether there is a comment or an empty line
@@ -798,4 +840,149 @@ void MetaDataTable::writeValueToString(std::string & result,
 
 
 
+void compareMetaDataTable(MetaDataTable &MD1, MetaDataTable &MD2,
+		MetaDataTable &MDboth, MetaDataTable &MDonly1, MetaDataTable &MDonly2,
+		EMDLabel label1, DOUBLE eps, EMDLabel label2, EMDLabel label3)
+{
+	if (!MD1.containsLabel(label1))
+		REPORT_ERROR("compareMetaDataTableEqualLabel::ERROR MD1 does not contain the specified label1.");
+	if (!MD2.containsLabel(label1))
+		REPORT_ERROR("compareMetaDataTableEqualLabel::ERROR MD2 does not contain the specified label1.");
+
+	if (label2 != EMDL_UNDEFINED)
+	{
+		if (!EMDL::isDouble(label1) || !EMDL::isDouble(label2))
+			REPORT_ERROR("compareMetaDataTableEqualLabel::ERROR 2D or 3D distances are only allowed for DOUBLEs.");
+		if (!MD1.containsLabel(label2))
+			REPORT_ERROR("compareMetaDataTableEqualLabel::ERROR MD1 does not contain the specified label2.");
+		if (!MD2.containsLabel(label2))
+			REPORT_ERROR("compareMetaDataTableEqualLabel::ERROR MD2 does not contain the specified label2.");
+	}
+
+	if (label3 != EMDL_UNDEFINED)
+	{
+		if (!EMDL::isDouble(label3))
+			REPORT_ERROR("compareMetaDataTableEqualLabel::ERROR 3D distances are only allowed for DOUBLEs.");
+		if (!MD1.containsLabel(label3))
+			REPORT_ERROR("compareMetaDataTableEqualLabel::ERROR MD1 does not contain the specified label3.");
+		if (!MD2.containsLabel(label3))
+			REPORT_ERROR("compareMetaDataTableEqualLabel::ERROR MD2 does not contain the specified label3.");
+	}
+
+	MDboth.clear();
+	MDonly1.clear();
+	MDonly2.clear();
+
+	std::string mystr1, mystr2;
+	int myint1, myint2;
+	DOUBLE myd1, myd2, mydy1 = 0., mydy2 = 0., mydz1 = 0., mydz2 = 0.;
+
+
+	// loop over MD1
+	std::vector<long int> to_remove_from_only2;
+	for (long int current_object1 = MD1.firstObject();
+	              current_object1 != MetaDataTable::NO_MORE_OBJECTS && current_object1 != MetaDataTable::NO_OBJECTS_STORED;
+	              current_object1 = MD1.nextObject())
+	{
+		if (EMDL::isString(label1))
+			MD1.getValue(label1, mystr1);
+		else if (EMDL::isInt(label1))
+			MD1.getValue(label1, myint1);
+		else if (EMDL::isDouble(label1))
+		{
+			MD1.getValue(label1, myd1);
+			if (label2 != EMDL_UNDEFINED)
+				MD1.getValue(label2, mydy1);
+			if (label3 != EMDL_UNDEFINED)
+				MD1.getValue(label3, mydz1);
+		}
+		else
+			REPORT_ERROR("compareMetaDataTableEqualLabel ERROR: only implemented for strings, integers or DOUBLEs");
+
+		// loop over MD2
+		bool have_in_2 = false;
+		for (long int current_object2 = MD2.firstObject();
+		              current_object2 != MetaDataTable::NO_MORE_OBJECTS && current_object2 != MetaDataTable::NO_OBJECTS_STORED;
+		              current_object2 = MD2.nextObject())
+		{
+
+			if (EMDL::isString(label1))
+			{
+				MD2.getValue(label1, mystr2);
+				if (strcmp(mystr1.c_str(), mystr2.c_str()) == 0)
+				{
+					have_in_2 = true;
+					to_remove_from_only2.push_back(current_object2);
+					MDboth.addObject(MD1.getObject());
+					break;
+				}
+			}
+			else if (EMDL::isInt(label1))
+			{
+				MD2.getValue(label1, myint2);
+				if ( ABS(myint2 - myint1) <= ROUND(eps) )
+				{
+					have_in_2 = true;
+					to_remove_from_only2.push_back(current_object2);
+					MDboth.addObject(MD1.getObject());
+					break;
+				}
+			}
+			else if (EMDL::isDouble(label1))
+			{
+				MD2.getValue(label1, myd2);
+				if (label2 != EMDL_UNDEFINED)
+					MD2.getValue(label2, mydy2);
+				if (label3 != EMDL_UNDEFINED)
+					MD2.getValue(label3, mydz2);
+
+				DOUBLE dist = sqrt( (myd1 - myd2) * (myd1 - myd2) +
+						            (mydy1 - mydy2) * (mydy1 - mydy2) +
+						            (mydz1 - mydz2) * (mydz1 - mydz2) );
+				if ( ABS(dist) <= eps )
+				{
+					have_in_2 = true;
+					to_remove_from_only2.push_back(current_object2);
+					//std::cerr << " current_object1= " << current_object1 << std::endl;
+					//std::cerr << " myd1= " << myd1 << " myd2= " << myd2 << " mydy1= " << mydy1 << " mydy2= " << mydy2 << " dist= "<<dist<<std::endl;
+					//std::cerr << " to be removed current_object2= " << current_object2 << std::endl;
+					MDboth.addObject(MD1.getObject());
+					break;
+				}
+			}
+		}
+
+		if (!have_in_2)
+		{
+			MDonly1.addObject(MD1.getObject());
+		}
+	}
+
+
+
+	for (long int current_object2 = MD2.firstObject();
+				current_object2 != MetaDataTable::NO_MORE_OBJECTS && current_object2 != MetaDataTable::NO_OBJECTS_STORED;
+				current_object2 = MD2.nextObject())
+	{
+
+		bool to_be_removed = false;
+		for (long int i = 0; i < to_remove_from_only2.size(); i++)
+		{
+			if (to_remove_from_only2[i] == current_object2)
+			{
+				to_be_removed = true;
+				break;
+			}
+		}
+		if (!to_be_removed)
+		{
+			//std::cerr << " doNOT remove current_object2= " << current_object2 << std::endl;
+			MDonly2.addObject(MD2.getObject(current_object2));
+		}
+	}
+
+
+}
+
+
 
