@@ -24,12 +24,12 @@ void MlModel::initialise()
 {
 
 	// Auxiliary vector with relevant size in Fourier space
-	MultidimArray<double > aux;
+	MultidimArray<DOUBLE > aux;
     aux.initZeros(ori_size / 2 + 1);
 
 	// Now resize all relevant vectors
     Iref.resize(nr_classes);
-    pdf_class.resize(nr_classes, 1./(double)nr_classes);
+    pdf_class.resize(nr_classes, 1./(DOUBLE)nr_classes);
     pdf_direction.resize(nr_classes);
     group_names.resize(nr_groups, "");
     sigma2_noise.resize(nr_groups, aux);
@@ -47,13 +47,13 @@ void MlModel::initialise()
 
 	if (ref_dim==2)
 	{
-		Matrix1D<double> empty(2);
+		Matrix1D<DOUBLE> empty(2);
 		prior_offset_class.resize(nr_classes, empty);
 	}
 	// These arrays will be resized when they are filled
 	orientability_contrib.resize(nr_classes);
 
-	Projector ref(ori_size, interpolator, padding_factor, r_min_nn);
+	Projector ref(ori_size, interpolator, padding_factor, r_min_nn, data_dim);
     PPref.clear();
     // Now fill the entire vector with instances of "ref"
     PPref.resize(nr_classes, ref);
@@ -98,6 +98,10 @@ void MlModel::read(FileName fn_in)
 		!MDlog.getValue(EMDL_MLMODEL_AVE_PMAX, ave_Pmax) )
 		REPORT_ERROR("MlModel::readStar: incorrect model_general table");
 
+    // Retain compability with model files written by Relion prior to 1.4
+    if (!MDlog.getValue(EMDL_MLMODEL_DIMENSIONALITY_DATA, data_dim))
+        data_dim=2;
+
     // Take inverse again of current resolution:
     current_resolution = 1. / current_resolution;
 
@@ -111,7 +115,7 @@ void MlModel::read(FileName fn_in)
 
 	// Read classes
 	FileName fn_tmp;
-	Image<double> img;
+	Image<DOUBLE> img;
 	MDclass.readStar(in, "model_classes");
 	int iclass = 0;
 	FOR_ALL_OBJECTS_IN_METADATA_TABLE(MDclass)
@@ -196,8 +200,8 @@ void MlModel::read(FileName fn_in)
 		{
 			MDclass.readStar(in, "model_pdf_orient_class_" + integerToString(iclass + 1));
 			pdf_direction[iclass].clear();
-			double aux;
-			std::vector<double> vaux;
+			DOUBLE aux;
+			std::vector<DOUBLE> vaux;
 			vaux.clear();
 			FOR_ALL_OBJECTS_IN_METADATA_TABLE(MDclass)
 			{
@@ -230,18 +234,18 @@ void MlModel::read(FileName fn_in)
 
 }
 
-void MlModel::write(FileName fn_out, HealpixSampling &sampling)
+void MlModel::write(FileName fn_out, HealpixSampling &sampling, bool do_write_bild)
 {
 
 	MetaDataTable MDclass, MDgroup, MDlog, MDsigma;
     FileName fn_tmp, fn_tmp2;
-    double aux;
+    DOUBLE aux;
     std::ofstream  fh;
 
     // A. Write images
     if (ref_dim == 2)
     {
-    	Image<double> img(XSIZE(Iref[0]), YSIZE(Iref[0]), 1, nr_classes);
+    	Image<DOUBLE> img(XSIZE(Iref[0]), YSIZE(Iref[0]), 1, nr_classes);
     	for (int iclass = 0; iclass < nr_classes; iclass++)
     	{
     		FOR_ALL_DIRECT_ELEMENTS_IN_ARRAY2D(Iref[iclass])
@@ -253,7 +257,7 @@ void MlModel::write(FileName fn_out, HealpixSampling &sampling)
     }
     else
     {
-    	Image<double> img;
+    	Image<DOUBLE> img;
     	// Set correct voxel size in the header
 		img.MDMainHeader.setValue(EMDL_IMAGE_SAMPLINGRATE_X, pixel_size);
 		img.MDMainHeader.setValue(EMDL_IMAGE_SAMPLINGRATE_Y, pixel_size);
@@ -267,16 +271,19 @@ void MlModel::write(FileName fn_out, HealpixSampling &sampling)
 
     	}
 
-    	// Also write out bild files with the orientational distribution of each class
-		// Also write out angular distributions
-		for (int iclass = 0; iclass < nr_classes; iclass++)
-		{
-			FileName fn_bild;
-			fn_bild.compose(fn_out+"_class",iclass+1,"", 3);
-			fn_bild += "_angdist.bild";
-			double offset = ori_size * pixel_size / 2.;
-			sampling.writeBildFileOrientationalDistribution(pdf_direction[iclass], fn_bild, offset, offset);
-		}
+    	if (do_write_bild)
+    	{
+			// Also write out bild files with the orientational distribution of each class
+			// Also write out angular distributions
+			for (int iclass = 0; iclass < nr_classes; iclass++)
+			{
+				FileName fn_bild;
+				fn_bild.compose(fn_out+"_class",iclass+1,"", 3);
+				fn_bild += "_angdist.bild";
+				DOUBLE offset = ori_size * pixel_size / 2.;
+				sampling.writeBildFileOrientationalDistribution(pdf_direction[iclass], fn_bild, offset, offset);
+			}
+    	}
 
 	}
 
@@ -291,6 +298,7 @@ void MlModel::write(FileName fn_out, HealpixSampling &sampling)
 	MDlog.addObject();
 	MDlog.setName("model_general");
 	MDlog.setValue(EMDL_MLMODEL_DIMENSIONALITY, ref_dim);
+	MDlog.setValue(EMDL_MLMODEL_DIMENSIONALITY_DATA, data_dim);
 	MDlog.setValue(EMDL_MLMODEL_ORIGINAL_SIZE, ori_size);
 	MDlog.setValue(EMDL_MLMODEL_CURRENT_RESOLUTION, 1./current_resolution);
 	MDlog.setValue(EMDL_MLMODEL_CURRENT_SIZE, current_size);
@@ -418,7 +426,7 @@ void MlModel::write(FileName fn_out, HealpixSampling &sampling)
 void  MlModel::readTauSpectrum(FileName fn_tau, int verb)
 {
 	MetaDataTable MDtau;
-	double val;
+	DOUBLE val;
 	int idx;
 	MDtau.read(fn_tau);
 	FOR_ALL_OBJECTS_IN_METADATA_TABLE(MDtau)
@@ -446,10 +454,13 @@ void MlModel::readImages(FileName fn_ref, int _ori_size, Experiment &_mydata,
 	// Set some stuff
 	nr_groups = _mydata.groups.size();
 	ori_size = _ori_size;
-	double avg_norm_correction = 1.;
+	DOUBLE avg_norm_correction = 1.;
+
+	// Data dimensionality
+	_mydata.MDexp.getValue(EMDL_IMAGE_DIMENSIONALITY, data_dim);
 
 	// Read references into memory
-	Image<double> img;
+	Image<DOUBLE> img;
 	FileName fn_tmp;
 	if (fn_ref != "None")
 	{
@@ -503,8 +514,13 @@ void MlModel::readImages(FileName fn_ref, int _ori_size, Experiment &_mydata,
 		do_average_unaligned = true;
 		do_generate_seeds = true;
 		refs_are_ctf_corrected = false;
-		img().initZeros(ori_size, ori_size);
-		ref_dim = 2;
+		if (data_dim == 2)
+			img().initZeros(ori_size, ori_size);
+		else if (data_dim == 3)
+			img().initZeros(ori_size, ori_size, ori_size);
+		else
+			REPORT_ERROR("MlOptimiser::read: ERROR data_dim should be either 2 or 3");
+		ref_dim = data_dim;
 		for (int iclass = 0; iclass < nr_classes; iclass++)
 			Iref.push_back(img());
 	}
@@ -569,27 +585,24 @@ void MlModel::expandToMovieFrames(Experiment &moviedata, int running_avg_side)
 
 	for (long int ipart = 0; ipart < moviedata.particles.size(); ipart++)
 	{
-		for (int iimg = 0; iimg < (moviedata.particles[ipart]).images.size(); iimg++)
-		{
-			long int group_id = ((moviedata.particles[ipart]).images[iimg]).group_id;
-			long int img_id = ((moviedata.particles[ipart]).images[iimg]).id;
-			// count the number of particles in this group
-			moviemodel.nr_particles_group[group_id]++;
-			// Get and check the number of frames is constant within one group
-			int nframes = -1;
-			if (!moviedata.MDimg.getValue(EMDL_PARTICLE_NR_FRAMES, nframes, img_id))
-				REPORT_ERROR("MlModel::expandToMovieFrames ERROR: cannot find rlnNrOfFrames in moviedata.");
-			if (nr_frames_in_group[group_id] < 0)
-				nr_frames_in_group[group_id] = nframes;
-			else if (nr_frames_in_group[group_id] != nframes)
-				REPORT_ERROR((std::string)"MlModel::expandToMovieFrames ERROR: unequal number of frames in group" + moviemodel.group_names[group_id]);
-		}
+		long int group_id = (moviedata.particles[ipart]).group_id;
+		long int part_id = (moviedata.particles[ipart]).id;
+		// count the number of particles in this group
+		moviemodel.nr_particles_group[group_id]++;
+		// Get and check the number of frames is constant within one group
+		int nframes = -1;
+		if (!moviedata.MDimg.getValue(EMDL_PARTICLE_NR_FRAMES, nframes, part_id))
+			REPORT_ERROR("MlModel::expandToMovieFrames ERROR: cannot find rlnNrOfFrames in moviedata.");
+		if (nr_frames_in_group[group_id] < 0)
+			nr_frames_in_group[group_id] = nframes;
+		else if (nr_frames_in_group[group_id] != nframes)
+			REPORT_ERROR((std::string)"MlModel::expandToMovieFrames ERROR: unequal number of frames in group" + moviemodel.group_names[group_id]);
 	}
 
 	// Correct the input sigma2noise spectra by a factor of nframes
 	for (int i=0; i < moviemodel.nr_groups; i++)
 	{
-		moviemodel.sigma2_noise[i] *= (double)nr_frames_in_group[i]/((double)(2 * running_avg_side + 1));
+		moviemodel.sigma2_noise[i] *= (DOUBLE)nr_frames_in_group[i]/((DOUBLE)(2 * running_avg_side + 1));
 	}
 
 	// Now replace the current model with the expanded moviemodel
@@ -608,7 +621,7 @@ void MlModel::initialisePdfDirection(int newsize)
 		if (oldsize == 0 || oldsize != newsize)
 		{
 			pdf_direction[iclass].resize(newsize);
-			pdf_direction[iclass].initConstant(1./((double) nr_classes * newsize));
+			pdf_direction[iclass].initConstant(1./((DOUBLE) nr_classes * newsize));
 		}
 	}
 	nr_directions = newsize;
@@ -626,7 +639,7 @@ void MlModel::setFourierTransformMaps(bool update_tau2_spectra, int nr_threads)
         }
         else
         {
-        	MultidimArray<double> dummy;
+        	MultidimArray<DOUBLE> dummy;
         	PPref[iclass].computeFourierTransformMap(Iref[iclass], dummy, current_size, nr_threads);
         }
     }
@@ -637,23 +650,23 @@ void MlModel::initialiseDataVersusPrior(bool fix_tau)
 {
 
     // Get total number of particles
-	double nr_particles = 0.;
+	DOUBLE nr_particles = 0.;
 	for (int igroup = 0; igroup < nr_particles_group.size(); igroup++)
-		nr_particles += (double)nr_particles_group[igroup];
+		nr_particles += (DOUBLE)nr_particles_group[igroup];
 
 	// Calculate average sigma2_noise over all image groups
-	MultidimArray<double> avg_sigma2_noise;
+	MultidimArray<DOUBLE> avg_sigma2_noise;
 	avg_sigma2_noise.initZeros(sigma2_noise[0]);
 	for (int igroup = 0; igroup < nr_particles_group.size(); igroup++)
 	{
-		avg_sigma2_noise += (double)(nr_particles_group[igroup]) * sigma2_noise[igroup];
+		avg_sigma2_noise += (DOUBLE)(nr_particles_group[igroup]) * sigma2_noise[igroup];
 	}
 	avg_sigma2_noise /= nr_particles;
 
 	// Get the FT of all reference structures
     // The Fourier Transforms are all "normalised" for 2D transforms of size = ori_size x ori_size
     // And spectrum is squared, so ori_size*ori_size in the 3D case!
-	double normfft = (ref_dim == 3) ? (double)(ori_size * ori_size) : 1.;
+	DOUBLE normfft = (ref_dim == 3 && data_dim == 2) ? (DOUBLE)(ori_size * ori_size) : 1.;
 
     for (int iclass = 0; iclass < nr_classes; iclass++)
 	{
@@ -661,7 +674,7 @@ void MlModel::initialiseDataVersusPrior(bool fix_tau)
 		tau2_class[iclass].resize(sigma2_noise[0]);
 
 		// Get the power spectrum of the reference
-		MultidimArray<double> spectrum(sigma2_noise[0]);
+		MultidimArray<DOUBLE> spectrum(sigma2_noise[0]);
 		getSpectrum(Iref[iclass], spectrum, POWER_SPECTRUM);
 
 		// Factor two because of two-dimensionality of the complex plane
@@ -683,12 +696,12 @@ void MlModel::initialiseDataVersusPrior(bool fix_tau)
 		fsc_halves_class[iclass].initZeros(sigma2_noise[0]);
 		FOR_ALL_DIRECT_ELEMENTS_IN_ARRAY1D(tau2_class[iclass])
 		{
-			double evidence = nr_particles * pdf_class[iclass] / DIRECT_A1D_ELEM(avg_sigma2_noise, i);
+			DOUBLE evidence = nr_particles * pdf_class[iclass] / DIRECT_A1D_ELEM(avg_sigma2_noise, i);
 			// empirical accounting for ratio of pixels in 3D shells compared to 2D shells
 			if (ref_dim == 3 && i > 0)
-				evidence /= (2. * (double)i);
-			double prior = 1. /  DIRECT_A1D_ELEM(tau2_class[iclass], i);
-			double myssnr = evidence / prior;
+				evidence /= (2. * (DOUBLE)i);
+			DOUBLE prior = 1. /  DIRECT_A1D_ELEM(tau2_class[iclass], i);
+			DOUBLE myssnr = evidence / prior;
 			DIRECT_A1D_ELEM(data_vs_prior_class[iclass], i ) = myssnr;
 			// Also initialise FSC-halves here (...)
 			//DIRECT_A1D_ELEM(fsc_halves_class[iclass], i ) = myssnr / (myssnr + 1);
@@ -704,6 +717,7 @@ void MlWsumModel::initialise(MlModel &_model, FileName fn_sym)
     nr_groups = _model.nr_groups;
     nr_directions = _model.nr_directions;
     ref_dim = _model.ref_dim;
+    data_dim = _model.data_dim;
     ori_size = _model.ori_size;
     pdf_class = _model.pdf_class;
     if (ref_dim == 2)
@@ -730,13 +744,13 @@ void MlWsumModel::initialise(MlModel &_model, FileName fn_sym)
     orientability_contrib.clear();
 
 
-    MultidimArray<double> aux(ori_size / 2 + 1);
+    MultidimArray<DOUBLE> aux(ori_size / 2 + 1);
     wsum_signal_product_spectra.resize(nr_groups, aux);
     wsum_reference_power_spectra.resize(nr_groups, aux);
 
     // Resize MlWsumModel-specific vectors
     BackProjector BP(ori_size, ref_dim, fn_sym, interpolator, padding_factor, r_min_nn,
-    		         ML_BLOB_ORDER, ML_BLOB_RADIUS, ML_BLOB_ALPHA);
+    		         ML_BLOB_ORDER, ML_BLOB_RADIUS, ML_BLOB_ALPHA, data_dim);
     BPref.clear();
     BPref.resize(nr_classes, BP);
     sumw_group.resize(nr_groups);
@@ -783,7 +797,7 @@ void MlWsumModel::initZeros()
 #define MAX_PACK_SIZE 671010000
 #endif
 
-void MlWsumModel::pack(MultidimArray<double> &packed)
+void MlWsumModel::pack(MultidimArray<DOUBLE> &packed)
 {
 	// for LL & avePmax & sigma2_offset & avg_norm_correction & sigma2_rot & sigma2_tilt & sigma2_psi
 	long long int packed_size = 0;
@@ -887,7 +901,7 @@ void MlWsumModel::pack(MultidimArray<double> &packed)
     }
 
 }
-void MlWsumModel::unpack(MultidimArray<double> &packed)
+void MlWsumModel::unpack(MultidimArray<DOUBLE> &packed)
 {
     int spectral_size = (ori_size / 2) + 1;
 
@@ -959,7 +973,7 @@ void MlWsumModel::unpack(MultidimArray<double> &packed)
 }
 
 
-void MlWsumModel::pack(MultidimArray<double> &packed, int &piece, int &nr_pieces, bool do_clear)
+void MlWsumModel::pack(MultidimArray<DOUBLE> &packed, int &piece, int &nr_pieces, bool do_clear)
 {
 
 
@@ -999,7 +1013,7 @@ void MlWsumModel::pack(MultidimArray<double> &packed, int &piece, int &nr_pieces
     {
         idx_start = piece * MAX_PACK_SIZE;
         idx_stop = XMIPP_MIN(idx_start + MAX_PACK_SIZE, packed_size);
-        nr_pieces = CEIL((double)packed_size/(double)MAX_PACK_SIZE);
+        nr_pieces = CEIL((DOUBLE)packed_size/(DOUBLE)MAX_PACK_SIZE);
     }
     else
     {
@@ -1126,7 +1140,7 @@ void MlWsumModel::pack(MultidimArray<double> &packed, int &piece, int &nr_pieces
 
 }
 
-void MlWsumModel::unpack(MultidimArray<double> &packed, int piece, bool do_clear)
+void MlWsumModel::unpack(MultidimArray<DOUBLE> &packed, int piece, bool do_clear)
 {
 
 
