@@ -21,6 +21,7 @@
 #ifndef ML_OPTIMISER_H_
 #define ML_OPTIMISER_H_
 
+#include <pthread.h>
 #include "src/ml_model.h"
 #include "src/parallel.h"
 #include "src/exp_model.h"
@@ -67,16 +68,6 @@
 #define METADATA_BEAMTILT_X 24
 #define METADATA_BEAMTILT_Y 25
 
-#define METADATA_MAT_0_0 26
-#define METADATA_MAT_0_1 27
-#define METADATA_MAT_0_2 28
-#define METADATA_MAT_1_0 29
-#define METADATA_MAT_1_1 30
-#define METADATA_MAT_1_2 31
-#define METADATA_MAT_2_0 32
-#define METADATA_MAT_2_1 33
-#define METADATA_MAT_2_2 34
-
 #define DO_WRITE_DATA true
 #define DONT_WRITE_DATA false
 #define DO_WRITE_SAMPLING true
@@ -130,7 +121,7 @@ public:
 	bool fix_tau;
 
     // some parameters for debugging
-	double debug1, debug2;
+	DOUBLE debug1, debug2;
 
 	// Starting and finishing particles (for parallelisation)
     long int my_first_ori_particle_id, my_last_ori_particle_id;
@@ -142,7 +133,7 @@ public:
 	bool do_split_random_halves;
 
 	// resolution (in Angstrom) to join the two random halves
-	double low_resol_join_halves;
+	DOUBLE low_resol_join_halves;
 
 	// Flag to join random halves again
 	bool do_join_random_halves;
@@ -184,7 +175,7 @@ public:
 	int nr_iter_wo_resol_gain;
 
 	// Best resolution obtained thus far
-	double best_resol_thus_far;
+	DOUBLE best_resol_thus_far;
 
 	// Is the FSC still high at the resolution limit?
 	bool has_high_fsc_at_limit;
@@ -196,22 +187,22 @@ public:
 	int autosampling_hporder_local_searches;
 
 	// Smallest changes thus far in the optimal translational offsets, orientations and classes
-	double smallest_changes_optimal_offsets;
-	double smallest_changes_optimal_orientations;
+	DOUBLE smallest_changes_optimal_offsets;
+	DOUBLE smallest_changes_optimal_orientations;
 	int smallest_changes_optimal_classes;
 
 	// Number of iterations without a decrease in OffsetChanges
 	int nr_iter_wo_large_hidden_variable_changes;
 
 	// Strict high-res limit in the expectation step
-	double strict_highres_exp;
+	DOUBLE strict_highres_exp;
 
 	// Flag to indicate to estimate angular accuracy until current_size (and not coarse_size) when restricting high-res limit in the expectation step
 	// This is used for testing purposes only
 	bool do_acc_currentsize_despite_highres_exp;
 
 	// Global parameters to store accuracy on rot and trans
-	double acc_rot, acc_trans;
+	DOUBLE acc_rot, acc_trans;
 
 	// Flag to indicate to use all data out to Nyquist
 	bool do_use_all_data;
@@ -235,7 +226,7 @@ public:
 	int max_coarse_size;
 
 	// Particle diameter (in Ang)
-	double particle_diameter;
+	DOUBLE particle_diameter;
 
 	// How many fourier shells should be included beyond the highest shell where evidenceVsPriorRatio < 1?
 	int incr_size;
@@ -257,10 +248,10 @@ public:
 	int width_mask_edge;
 
 	// Number of particles to be processed simultaneously
-	int nr_pool, max_nr_pool;
+	int nr_pool;
 
 	// Available memory (in Gigabyte)
-	double available_memory;
+	DOUBLE available_memory;
 
 	// Perform combination of weight through files written on disc
 	bool combine_weights_thru_disc;
@@ -268,6 +259,12 @@ public:
 	// Only print metadata label definitions and exit
 	bool do_print_metadata_labels;
 
+        // Use parallel access to disc?
+        bool do_parallel_disc_io;
+
+        // Or preread all images into RAM on the master node?
+        bool do_preread_images;
+
 	// Print the symmetry transformation matrices
 	bool do_print_symmetry_ops;
 
@@ -290,7 +287,7 @@ public:
 	 * The closer to one, the more orientations will be oversampled
 	 * The default is 0.999.
 	 */
-	double adaptive_fraction;
+	DOUBLE adaptive_fraction;
 
 	// Seed for random number generator
 	int random_seed;
@@ -313,7 +310,7 @@ public:
 	bool do_always_cc;
 
 	// Initial low-pass filter for all references (in digital frequency)
-	double ini_high;
+	DOUBLE ini_high;
 
 	// Flag whether to generate seeds
 	// TODO: implement!
@@ -358,7 +355,7 @@ public:
 	FileName fn_sigma;
 
 	// Multiplicative fdge factor for the sigma estimates
-	double sigma2_fudge;
+	DOUBLE sigma2_fudge;
 
 	// Perform two reconstructions of random halves sequentially (to save memory for very big cases)
 	bool do_sequential_halves_recons;
@@ -368,24 +365,19 @@ public:
 	// Until now the best refinements have used the noisy mask, not the soft mask....
 	bool do_zero_mask;
 
-
-	// Developmental: simulated annealing to get out of local minima...
-	bool do_sim_anneal;
-	double temperature, temp_ini, temp_fin;
-
 	/////////// Keep track of hidden variable changes ////////////////////////
 
 	// Changes from one iteration to the next in the angles
-	double current_changes_optimal_orientations, sum_changes_optimal_orientations;
+	DOUBLE current_changes_optimal_orientations, sum_changes_optimal_orientations;
 
 	// Changes from one iteration to the next in the translations
-	double current_changes_optimal_offsets, sum_changes_optimal_offsets;
+	DOUBLE current_changes_optimal_offsets, sum_changes_optimal_offsets;
 
 	// Changes from one iteration to the next in the class assignments
-	double current_changes_optimal_classes, sum_changes_optimal_classes;
+	DOUBLE current_changes_optimal_classes, sum_changes_optimal_classes;
 
 	// Just count how often the optimal changes are summed
-	double sum_changes_count;
+	DOUBLE sum_changes_count;
 
 	/////////// Some internal stuff ////////////////////////
 
@@ -396,49 +388,59 @@ public:
 	TabSine tab_sin;
 	TabCosine tab_cos;
 
-	// Loop from iclass_min to iclass_max to deal with seed generation in the first iteration
-	int iclass_min, iclass_max;
-
 	// Verbosity flag
 	int verb;
 
-	// Thread Managers for the expectation step: one for allorientations, the other for all (pooled) particles
-	ThreadTaskDistributor *exp_iorient_ThreadTaskDistributor, *exp_ipart_ThreadTaskDistributor;
+	// Thread Managers for the expectation step: one for all (pooled) particles
+	ThreadTaskDistributor *exp_ipart_ThreadTaskDistributor;
 
 	// Number of threads to run in parallel
-	int nr_threads, nr_threads_original;
+	int nr_threads;
+
+	long int exp_my_first_ori_particle, exp_my_last_ori_particle;
+	MultidimArray<DOUBLE> exp_metadata, exp_imagedata;
+	std::string exp_fn_img, exp_fn_ctf, exp_fn_recimg;
+	int exp_nr_images;
+
+	// Calculate translated images on-the-fly
+	bool do_shifts_onthefly;
+	std::vector<MultidimArray<Complex> > global_fftshifts_ab_coarse, global_fftshifts_ab_current, global_fftshifts_ab2_coarse, global_fftshifts_ab2_current;
 
 	/** Some global variables that are only for thread visibility */
 	/// Taken from getAllSquaredDifferences
+	/*
 	std::vector<MultidimArray<Complex > > exp_Fimgs, exp_Fimgs_nomask, exp_local_Fimgs_shifted, exp_local_Fimgs_shifted_nomask;
-	std::vector<MultidimArray<double> > exp_Fctfs, exp_local_Fctfs, exp_local_Minvsigma2s;
-	Matrix2D<double> exp_R_mic;
+	std::vector<MultidimArray<DOUBLE> > exp_Fctfs, exp_local_Fctfs, exp_local_Minvsigma2s;
+	Matrix2D<DOUBLE> exp_R_mic;
 	int exp_iseries, exp_iclass, exp_ipass, exp_iimage, exp_ipart, exp_current_image_size, exp_current_oversampling, exp_nr_ori_particles, exp_nr_particles, exp_nr_images;
 	long int exp_nr_oversampled_rot, exp_nr_oversampled_trans, exp_nr_rot, exp_nr_dir, exp_nr_psi, exp_nr_trans;
-	long int exp_part_id, exp_my_first_ori_particle, exp_my_last_ori_particle;
+	long int exp_part_id, exp_my_first_ori_particle, exp_my_last_ori_particle
 	std::vector<int> exp_starting_image_no;
 	std::vector<long int> exp_ipart_to_part_id, exp_ipart_to_ori_part_id, exp_ipart_to_ori_part_nframe, exp_iimg_to_ipart;
-	std::vector<double> exp_highres_Xi2_imgs, exp_min_diff2, exp_local_sqrtXi2, exp_local_oldcc;
-	MultidimArray<double> exp_Mweight;
+	std::vector<DOUBLE> exp_highres_Xi2_imgs, exp_min_diff2, exp_local_sqrtXi2, exp_local_oldcc;
+	MultidimArray<DOUBLE> exp_Mweight;
 	MultidimArray<bool> exp_Mcoarse_significant;
 	// And from storeWeightedSums
-	std::vector<double> exp_sum_weight, exp_significant_weight, exp_max_weight;
-	std::vector<Matrix1D<double> > exp_old_offset, exp_prior;
-	std::vector<double> exp_wsum_norm_correction;
-	std::vector<MultidimArray<double> > exp_wsum_scale_correction_XA, exp_wsum_scale_correction_AA, exp_power_imgs;
-	MultidimArray<double> exp_metadata, exp_imagedata;
-	double exp_thisparticle_sumweight;
-
+	std::vector<DOUBLE> exp_sum_weight, exp_significant_weight, exp_max_weight;
+	std::vector<Matrix1D<DOUBLE> > exp_old_offset, exp_prior;
+	std::vector<DOUBLE> exp_wsum_norm_correction;
+	std::vector<MultidimArray<DOUBLE> > exp_wsum_scale_correction_XA, exp_wsum_scale_correction_AA, exp_power_imgs;
+	DOUBLE exp_thisparticle_sumweight;
+	*/
 	//TMP DEBUGGING
-	MultidimArray<double> DEBUGGING_COPY_exp_Mweight;
+	MultidimArray<DOUBLE> DEBUGGING_COPY_exp_Mweight;
 
 #ifdef TIMING
     Timer timer;
 	int TIMING_DIFF_PROJ, TIMING_DIFF_SHIFT, TIMING_DIFF_DIFF2;
 	int TIMING_WSUM_PROJ, TIMING_WSUM_BACKPROJ, TIMING_WSUM_DIFF2, TIMING_WSUM_SUMSHIFT;
 	int TIMING_EXP, TIMING_MAX, TIMING_RECONS;
-	int TIMING_ESP, TIMING_ESP_READ, TIMING_ESP_DIFF1, TIMING_ESP_DIFF2;
+	int TIMING_ESP, TIMING_ESP_THR, TIMING_ESP_ONEPART, TIMING_ESP_ONEPARTN, TIMING_EXP_METADATA, TIMING_EXP_CHANGES;
+	int TIMING_ESP_FT, TIMING_ESP_INI, TIMING_ESP_DIFF1, TIMING_ESP_DIFF2;
+	int TIMING_ESP_DIFF2_A, TIMING_ESP_DIFF2_B, TIMING_ESP_DIFF2_C, TIMING_ESP_DIFF2_D, TIMING_ESP_DIFF2_E;
+	int TIMING_ESP_PREC1, TIMING_ESP_PREC2, TIMING_ESP_PRECW, TIMING_WSUM_GETSHIFT, TIMING_DIFF2_GETSHIFT, TIMING_WSUM_SCALE, TIMING_WSUM_LOCALSUMS;
 	int TIMING_ESP_WEIGHT1, TIMING_ESP_WEIGHT2, TIMING_WEIGHT_EXP, TIMING_WEIGHT_SORT, TIMING_ESP_WSUM;
+	int TIMING_EXTRA1, TIMING_EXTRA2, TIMING_EXTRA3;
 #endif
 
 public:
@@ -476,13 +478,13 @@ public:
 	/* Calculates the sum of all individual power spectra and the average of all images for initial sigma_noise estimation
 	 * The rank is passed so that if one splits the data into random halves one can know which random half to treat
 	 */
-	void calculateSumOfPowerSpectraAndAverageImage(MultidimArray<double> &Mavg, bool myverb = true);
+	void calculateSumOfPowerSpectraAndAverageImage(MultidimArray<DOUBLE> &Mavg, bool myverb = true);
 
 	/** Use the sum of the individual power spectra to calculate their average and set this in sigma2_noise
 	 * Also subtract the power spectrum of the average images,
 	 * and if (do_average_unaligned) then also set Mavg to all Iref
 	 */
-	void setSigmaNoiseEstimatesAndSetAverageImage(MultidimArray<double> &Mavg);
+	void setSigmaNoiseEstimatesAndSetAverageImage(MultidimArray<DOUBLE> &Mavg);
 
 	/* Perform an initial low-pass filtering of the references
 	 * Note that because of the MAP estimation, this is not necessary inside the refinement
@@ -510,11 +512,20 @@ public:
 	/* Check whether everything fits into memory, possibly adjust nr_pool and setup thread task managers */
 	void expectationSetupCheckMemory(bool myverb = true);
 
+	/* For on-the-fly shifts, precalculates AB-matrices */
+	void precalculateABMatrices();
+
 	/* Perform the expectation integration over all k, phi and series elements for a number (some) of pooled particles
 	 * The number of pooled particles is determined by max_nr_pool and some memory checks in expectationSetup()
 	 */
 	void expectationSomeParticles(long int my_first_particle, long int my_last_particle);
 
+	/* Perform expectation step for some particles using threads */
+	void doThreadExpectationSomeParticles(int thread_id);
+
+	/* Perform the expectation integration over all k, phi and series elements for a given particle */
+	void expectationOneParticle(long int my_ori_particle, int thread_id);
+
 	/* Maximization step
 	 * Updates the current model: reconstructs and updates all other model parameter
 	 */
@@ -546,45 +557,103 @@ public:
 	 */
 	void updateImageSizeAndResolutionPointers();
 
-	/* Calculates the PDF of the in-plane translation
-	 * assuming a 2D (or 3D) Gaussian centered at (0,0) and with stddev of mymodel.sigma2_offset
-	 */
-	double calculatePdfOffset(Matrix1D<double> offset, Matrix1D<double> prior);
-
 	/* From the vectors of Fourier transforms of the images, calculate running averages over the movie frames
 	 */
-	void calculateRunningAveragesOfMovieFrames();
+	void calculateRunningAveragesOfMovieFrames(long int my_ori_particle,
+		std::vector<MultidimArray<Complex > > &exp_Fimgs,
+		std::vector<MultidimArray<DOUBLE> > &exp_power_imgs,
+		std::vector<DOUBLE> &exp_highres_Xi2_imgs);
 
 	/* Read image and its metadata from disc (threaded over all pooled particles)
 	 */
-	void doThreadGetFourierTransformsAndCtfs(int thread_id);
+	void getFourierTransformsAndCtfs(long int my_ori_particle, int metadata_offset,
+			std::vector<MultidimArray<Complex > > &exp_Fimgs,
+			std::vector<MultidimArray<Complex > > &exp_Fimgs_nomask,
+			std::vector<MultidimArray<DOUBLE> > &exp_Fctfs,
+			std::vector<Matrix1D<DOUBLE> > &exp_old_offset,
+			std::vector<Matrix1D<DOUBLE> > &exp_prior,
+			std::vector<MultidimArray<DOUBLE> > &exp_power_imgs,
+			std::vector<DOUBLE> &exp_highres_Xi2_imgs,
+			std::vector<int> &exp_pointer_dir_nonzeroprior,
+			std::vector<int> &exp_pointer_psi_nonzeroprior,
+			std::vector<DOUBLE> &exp_directions_prior,
+			std::vector<DOUBLE> &exp_psi_prior);
 
 	/* Store all shifted FourierTransforms in a vector
 	 * also store precalculated 2D matrices with 1/sigma2_noise
 	 */
-	void doThreadPrecalculateShiftedImagesCtfsAndInvSigma2s(int thread_id);
+	void precalculateShiftedImagesCtfsAndInvSigma2s(bool do_also_unmasked, long int my_ori_particle,
+			int exp_current_image_size, int exp_current_oversampling,
+			int exp_itrans_min, int exp_itrans_max,
+			std::vector<MultidimArray<Complex > > &exp_Fimgs,
+			std::vector<MultidimArray<Complex > > &exp_Fimgs_nomask,
+			std::vector<MultidimArray<DOUBLE> > &exp_Fctfs,
+			std::vector<MultidimArray<Complex > > &exp_local_Fimgs_shifted,
+			std::vector<MultidimArray<Complex > > &exp_local_Fimgs_shifted_nomask,
+			std::vector<MultidimArray<DOUBLE> > &exp_local_Fctfs,
+			std::vector<DOUBLE> &exp_local_sqrtXi2,
+			std::vector<MultidimArray<DOUBLE> > &exp_local_Minvsigma2s);
 
 	// Given exp_Mcoarse_significant, check for iorient whether any of the particles has any significant (coarsely sampled) translation
-	bool isSignificantAnyParticleAnyTranslation(long int iorient);
-
-	// Threaded core of getAllSquaredDifferences, where loops over all orientations are parallelised using threads
-	void doThreadGetSquaredDifferencesAllOrientations(int thread_id);
+	bool isSignificantAnyParticleAnyTranslation(long int iorient,
+			int exp_itrans_min, int exp_itrans_max, MultidimArray<bool> &exp_Mcoarse_significant);
 
 	// Get squared differences for all iclass, idir, ipsi and itrans...
-	void getAllSquaredDifferences();
-
-	// Threaded core of convertAllSquaredDifferencesToWeights, where loops over all orientations are parallelised using threads
-	void doThreadConvertSquaredDifferencesToWeightsAllOrientations(int thread_id);
+	void getAllSquaredDifferences(long int my_ori_particle, int exp_current_image_size,
+			int exp_ipass, int exp_current_oversampling, int metadata_offset,
+			int exp_idir_min, int exp_idir_max, int exp_ipsi_min, int exp_ipsi_max,
+			int exp_itrans_min, int exp_itrans_max, int my_iclass_min, int my_iclass_max,
+			std::vector<DOUBLE> &exp_min_diff2,
+			std::vector<DOUBLE> &exp_highres_Xi2_imgs,
+			std::vector<MultidimArray<Complex > > &exp_Fimgs,
+			std::vector<MultidimArray<DOUBLE> > &exp_Fctfs,
+			MultidimArray<DOUBLE> &exp_Mweight,
+			MultidimArray<bool> &exp_Mcoarse_significant,
+			std::vector<int> &exp_pointer_dir_nonzeroprior, std::vector<int> &exp_pointer_psi_nonzeroprior,
+			std::vector<DOUBLE> &exp_directions_prior, std::vector<DOUBLE> &exp_psi_prior,
+			std::vector<MultidimArray<Complex > > &exp_local_Fimgs_shifted,
+			std::vector<MultidimArray<DOUBLE> > &exp_local_Minvsigma2s,
+			std::vector<MultidimArray<DOUBLE> > &exp_local_Fctfs,
+			std::vector<DOUBLE> &exp_local_sqrtXi2);
 
 	// Convert all squared difference terms to weights.
 	// Also calculates exp_sum_weight and, for adaptive approach, also exp_significant_weight
-	void convertAllSquaredDifferencesToWeights();
-
-	// Threaded core of storeWeightedSums, where loops over all orientations are parallelised using threads
-	void doThreadStoreWeightedSumsAllOrientations(int thread_id);
+	void convertAllSquaredDifferencesToWeights(long int my_ori_particle, int exp_ipass,
+			int exp_current_oversampling, int metadata_offset,
+			int exp_idir_min, int exp_idir_max, int exp_ipsi_min, int exp_ipsi_max,
+			int exp_itrans_min, int exp_itrans_max, int my_iclass_min, int my_iclass_max,
+			MultidimArray<DOUBLE> &exp_Mweight, MultidimArray<bool> &exp_Mcoarse_significant,
+			std::vector<DOUBLE> &exp_significant_weight, std::vector<DOUBLE> &exp_sum_weight,
+			std::vector<Matrix1D<DOUBLE> > &exp_old_offset, std::vector<Matrix1D<DOUBLE> > &exp_prior,
+			std::vector<DOUBLE> &exp_min_diff2,
+			std::vector<int> &exp_pointer_dir_nonzeroprior, std::vector<int> &exp_pointer_psi_nonzeroprior,
+			std::vector<DOUBLE> &exp_directions_prior, std::vector<DOUBLE> &exp_psi_prior);
 
 	// Store all relevant weighted sums, also return optimal hidden variables, max_weight and dLL
-	void storeWeightedSums();
+	void storeWeightedSums(long int my_ori_particle, int exp_current_image_size,
+			int exp_current_oversampling, int metadata_offset,
+			int exp_idir_min, int exp_idir_max, int exp_ipsi_min, int exp_ipsi_max,
+			int exp_itrans_min, int exp_itrans_max, int my_iclass_min, int my_iclass_max,
+			std::vector<DOUBLE> &exp_min_diff2,
+			std::vector<DOUBLE> &exp_highres_Xi2_imgs,
+			std::vector<MultidimArray<Complex > > &exp_Fimgs,
+			std::vector<MultidimArray<Complex > > &exp_Fimgs_nomask,
+			std::vector<MultidimArray<DOUBLE> > &exp_Fctfs,
+			std::vector<MultidimArray<DOUBLE> > &exp_power_imgs,
+			std::vector<Matrix1D<DOUBLE> > &exp_old_offset,
+			std::vector<Matrix1D<DOUBLE> > &exp_prior,
+			MultidimArray<DOUBLE> &exp_Mweight,
+			MultidimArray<bool> &exp_Mcoarse_significant,
+			std::vector<DOUBLE> &exp_significant_weight,
+			std::vector<DOUBLE> &exp_sum_weight,
+			std::vector<DOUBLE> &exp_max_weight,
+			std::vector<int> &exp_pointer_dir_nonzeroprior, std::vector<int> &exp_pointer_psi_nonzeroprior,
+			std::vector<DOUBLE> &exp_directions_prior, std::vector<DOUBLE> &exp_psi_prior,
+			std::vector<MultidimArray<Complex > > &exp_local_Fimgs_shifted,
+			std::vector<MultidimArray<Complex > > &exp_local_Fimgs_shifted_nomask,
+			std::vector<MultidimArray<DOUBLE> > &exp_local_Minvsigma2s,
+			std::vector<MultidimArray<DOUBLE> > &exp_local_Fctfs,
+			std::vector<DOUBLE> &exp_local_sqrtXi2);
 
 	/** Monitor the changes in the optimal translations, orientations and class assignments for some particles */
 	void monitorHiddenVariableChanges(long int my_first_ori_particle, long int my_last_ori_particle);
@@ -613,19 +682,7 @@ public:
 
 };
 
-// Global call to threaded core of doThreadGetFourierTransformsAndCtfs
-void globalThreadGetFourierTransformsAndCtfs(ThreadArgument &thArg);
-
-// Global call to threaded core of doPrecalculateShiftedImagesCtfsAndInvSigma2s
-void globalThreadPrecalculateShiftedImagesCtfsAndInvSigma2s(ThreadArgument &thArg);
-
-// Global call to threaded core of getAllSquaredDifferences, where loops over all orientations are parallelised using threads
-void globalThreadGetSquaredDifferencesAllOrientations(ThreadArgument &thArg);
-
-// Global call to threaded core of convertAllSquaredDifferencesToWeights, where loops over all orientations are parallelised using threads
-void globalThreadConvertSquaredDifferencesToWeightsAllOrientations(ThreadArgument &thArg);
-
-// Global call to threaded core of convertAllSquaredDifferencesToWeights, where loops over all orientations are parallelised using threads
-void globalThreadConvertSquaredDifferencesToWeightsAllOrientations(ThreadArgument &thArg);
+// Global call to threaded core of doThreadExpectationSomeParticles
+void globalThreadExpectationSomeParticles(ThreadArgument &thArg);
 
 #endif /* MAXLIK_H_ */
