@@ -51,6 +51,7 @@ void Preprocessing::read(int argc, char **argv, int rank)
 	scale  = textToInteger(parser.getOption("--scale", "Re-scale the particles to this size (in pixels)", "-1"));
 	window  = textToInteger(parser.getOption("--window", "Re-window the particles to this size (in pixels)", "-1"));
 	do_normalise = parser.checkOption("--norm", "Normalise the background to average zero and stddev one");
+	do_ramp = !parser.checkOption("--no_ramp", "Just subtract the background mean in the normalisation, instead of subtracting a fitted ramping background. ");
 	bg_radius = textToInteger(parser.getOption("--bg_radius", "Radius of the circular mask that will be used to define the background area (in pixels)", "-1"));
 	white_dust_stddev = textToFloat(parser.getOption("--white_dust", "Sigma-values above which white dust will be removed (negative value means no dust removal)","-1"));
 	black_dust_stddev = textToFloat(parser.getOption("--black_dust", "Sigma-values above which black dust will be removed (negative value means no dust removal)","-1"));
@@ -176,7 +177,7 @@ void Preprocessing::joinAllStarFiles()
 
 	std::cout << " Joining all metadata in one STAR file..." << std::endl;
 	bool has_other_ctfs, has_this_ctf;
-	double defU, defV, defAng, CC, HT, CS, AmpCnst, XMAG, DStep;
+	DOUBLE defU, defV, defAng, CC, HT, CS, AmpCnst, XMAG, DStep;
 	has_other_ctfs = false;
 	FileName prev_fn_mic="";
 	for (long int ipos = 0; ipos < fn_coords.size(); ipos++)
@@ -190,7 +191,7 @@ void Preprocessing::joinAllStarFiles()
 		{
 			FileName fn_microot = getRootNameFromMicrographName(fn_mic);
 			// Gather the results from ctffind
-			has_this_ctf = getCtffind3Results(fn_microot, defU, defV, defAng, CC,
+			has_this_ctf = getCtffindResults(fn_microot, defU, defV, defAng, CC,
 					HT, CS, AmpCnst, XMAG, DStep);
 
 		}
@@ -198,7 +199,7 @@ void Preprocessing::joinAllStarFiles()
 
 			// Re-scaled detector pixel size
 		if (has_this_ctf && do_rescale)
-                    DStep *= (double)extract_size/(double)scale;
+                    DStep *= (DOUBLE)extract_size/(DOUBLE)scale;
 
 		if (ipos == 0 && has_this_ctf)
 		{
@@ -217,10 +218,10 @@ void Preprocessing::joinAllStarFiles()
 		}
 
 		if (!has_this_ctf && has_other_ctfs)
-			REPORT_ERROR("joinAllStarFiles%ERROR: Exiting because of missing CTFFIND3 logfiles for micrograph " + fn_mic);
+			REPORT_ERROR("joinAllStarFiles%ERROR: Exiting because of missing CTFFIND logfiles for micrograph " + fn_mic);
 
 		if (has_this_ctf && !has_other_ctfs)
-			REPORT_ERROR("joinAllStarFiles%ERROR: Exiting because of missing CTFFIND3 logfiles ...");
+			REPORT_ERROR("joinAllStarFiles%ERROR: Exiting because of missing CTFFIND logfiles ...");
 
 
 		FileName fn_star = "Particles/" + fn_mic.withoutExtension() + "_" + fn_out + ".star";
@@ -228,7 +229,7 @@ void Preprocessing::joinAllStarFiles()
 
 		FOR_ALL_OBJECTS_IN_METADATA_TABLE(MDonestack)
 		{
-			// This was double, right?
+			// This was DOUBLE, right?
 			//MDonestack.setValue(EMDL_MICROGRAPH_NAME, fn_mic);
 
 			if (has_this_ctf)
@@ -349,8 +350,8 @@ void Preprocessing::readCoordinates(FileName fn_coord, MetaDataTable &MD)
 				int ypos = num2 + num4 / 2;
 
 				MD.addObject();
-				MD.setValue(EMDL_IMAGE_COORD_X, (double)xpos);
-				MD.setValue(EMDL_IMAGE_COORD_Y, (double)ypos);
+				MD.setValue(EMDL_IMAGE_COORD_X, (DOUBLE)xpos);
+				MD.setValue(EMDL_IMAGE_COORD_Y, (DOUBLE)ypos);
 			}
 			else // Try reading as plain ASCII....
 			{
@@ -366,12 +367,12 @@ void Preprocessing::readCoordinates(FileName fn_coord, MetaDataTable &MD)
 					if (words.size() > 1 && sscanf(words[0].c_str(), "%d", &num1) && sscanf(words[1].c_str(), "%d", &num2))
 					{
 						MD.addObject();
-						MD.setValue(EMDL_IMAGE_COORD_X, (double)num1);
-						MD.setValue(EMDL_IMAGE_COORD_Y, (double)num2);
+						MD.setValue(EMDL_IMAGE_COORD_X, (DOUBLE)num1);
+						MD.setValue(EMDL_IMAGE_COORD_Y, (DOUBLE)num2);
 
 						// It could also be a X,Y,Z coordinate...
 						if (words.size() > 2 && sscanf(words[2].c_str(), "%d", &num3))
-							MD.setValue(EMDL_IMAGE_COORD_Z, (double)num3);
+							MD.setValue(EMDL_IMAGE_COORD_Z, (DOUBLE)num3);
 					}
 
 
@@ -388,11 +389,11 @@ void Preprocessing::readCoordinates(FileName fn_coord, MetaDataTable &MD)
 					int num3;
 
 					MD.addObject();
-					MD.setValue(EMDL_IMAGE_COORD_X, (double)num1);
-					MD.setValue(EMDL_IMAGE_COORD_Y, (double)num2);
+					MD.setValue(EMDL_IMAGE_COORD_X, (DOUBLE)num1);
+					MD.setValue(EMDL_IMAGE_COORD_Y, (DOUBLE)num2);
 					// It could also be a X,Y,Z coordinate...
 					if (words.size() > 2 && sscanf(words[2].c_str(), "%d", &num3))
-						MD.setValue(EMDL_IMAGE_COORD_Z, (double)num3);
+						MD.setValue(EMDL_IMAGE_COORD_Z, (DOUBLE)num3);
 
 				}
 			}
@@ -415,7 +416,7 @@ void Preprocessing::extractParticlesFromFieldOfView(FileName fn_coord)
 	{
 		FOR_ALL_OBJECTS_IN_METADATA_TABLE(MDin)
 		{
-			double xcoor, ycoor;
+			DOUBLE xcoor, ycoor;
 			MDin.getValue(EMDL_IMAGE_COORD_X, xcoor);
 			MDin.getValue(EMDL_IMAGE_COORD_Y, ycoor);
 			xcoor += extract_bias_x;
@@ -443,13 +444,15 @@ void Preprocessing::extractParticlesFromFieldOfView(FileName fn_coord)
 	}
 
 	// Read the header of the micrograph to see how many frames there are.
-	Image<double> Imic;
+	Image<DOUBLE> Imic;
 	Imic.read(fn_mic, false, -1, false); // readData = false, select_image = -1, mapData= false, is_2D = true);
 
 	int xdim, ydim, zdim;
 	long int ndim;
 	Imic.getDimensions(xdim, ydim, zdim, ndim);
 	dimensionality = (zdim > 1) ? 3 : 2;
+	if (dimensionality == 3 || do_movie_extract )
+            do_ramp = false;
 
 	// Name of the output stack
 	// Add the same root as the output STAR file (that way one could extract two "families" of different particle stacks)
@@ -462,10 +465,10 @@ void Preprocessing::extractParticlesFromFieldOfView(FileName fn_coord)
 		std::cout << "WARNING: movie " << fn_mic << " does not have multiple frames..." << std::endl;
 
 	long int my_current_nr_images = 0;
-	double all_avg = 0;
-	double all_stddev = 0;
-	double all_minval = 99.e99;
-	double all_maxval = -99.e99;
+	DOUBLE all_avg = 0;
+	DOUBLE all_stddev = 0;
+	DOUBLE all_minval = 99.e99;
+	DOUBLE all_maxval = -99.e99;
 
 	// To deal with default movie_last_frame value
 	if (movie_last_frame < 0)
@@ -496,10 +499,10 @@ void Preprocessing::extractParticlesFromFieldOfView(FileName fn_coord)
 void Preprocessing::extractParticlesFromOneFrame(MetaDataTable &MD,
 		FileName fn_mic, int iframe, int n_frames,
 		FileName fn_output_img_root, long int &my_current_nr_images, long int my_total_nr_images,
-		double &all_avg, double &all_stddev, double &all_minval, double &all_maxval)
+		DOUBLE &all_avg, DOUBLE &all_stddev, DOUBLE &all_minval, DOUBLE &all_maxval)
 {
 
-	Image<double> Ipart, Imic, Itmp;
+	Image<DOUBLE> Ipart, Imic, Itmp;
 
 	FileName fn_frame;
 	// If movies, then average over avg_n_frames
@@ -509,8 +512,8 @@ void Preprocessing::extractParticlesFromOneFrame(MetaDataTable &MD,
 		{
 			int iiframe = iframe + ii;
 			// If we run over the size of the movie, then discard these frames
-			if (iiframe >= n_frames)
-				return;
+			if (iiframe >= movie_first_frame + n_frames)
+				break;
 			fn_frame.compose(iiframe + 1, fn_mic);
 			if (ii==0)
 			{
@@ -534,7 +537,7 @@ void Preprocessing::extractParticlesFromOneFrame(MetaDataTable &MD,
 	int ipos = 0;
 	FOR_ALL_OBJECTS_IN_METADATA_TABLE(MD)
 	{
-		double dxpos, dypos, dzpos;
+		DOUBLE dxpos, dypos, dzpos;
 		long int xpos, ypos, zpos;
 		long int x0, xF, y0, yF, z0, zF;
 		MD.getValue(EMDL_IMAGE_COORD_X, dxpos);
@@ -617,7 +620,7 @@ void Preprocessing::extractParticlesFromOneFrame(MetaDataTable &MD,
 			if (dimensionality == 3 && do_project_3d)
 			{
 				// Project the 3D sub-tomogram into a 2D particle again
-				Image<double> Iproj(YSIZE(Ipart()), XSIZE(Ipart()));
+				Image<DOUBLE> Iproj(YSIZE(Ipart()), XSIZE(Ipart()));
 				Iproj().setXmippOrigin();
 				FOR_ALL_DIRECT_ELEMENTS_IN_ARRAY3D(Ipart())
 				{
@@ -643,7 +646,7 @@ void Preprocessing::extractParticlesFromOneFrame(MetaDataTable &MD,
 				fn_part.compose(ipos + 1,  "Particles/" + getRootNameFromMicrographName(fn_mic)); // start image counting in stacks at 1!
 				// for automated re-alignment of particles in relion_refine: have rlnParticleName equal to rlnImageName in non-movie star file
 				fn_part += "_" + fn_out + ".mrcs";
-				MD.setValue(EMDL_PARTICLE_NAME, fn_part);
+				MD.setValue(EMDL_PARTICLE_ORI_NAME, fn_part);
 			}
 			MD.setValue(EMDL_IMAGE_NAME, fn_img);
 			MD.setValue(EMDL_MICROGRAPH_NAME, fn_frame);
@@ -657,7 +660,7 @@ void Preprocessing::extractParticlesFromOneFrame(MetaDataTable &MD,
 
 void Preprocessing::runOperateOnInputFile(FileName fn_operate_on)
 {
-	Image<double> Ipart, Iout;
+	Image<DOUBLE> Ipart, Iout;
 	MetaDataTable MD;
 	long int Nimg;
 
@@ -682,10 +685,10 @@ void Preprocessing::runOperateOnInputFile(FileName fn_operate_on)
 		Nimg = NSIZE(Iout());
 	}
 
-	double all_avg = 0;
-	double all_stddev = 0;
-	double all_minval = 99.e99;
-	double all_maxval = -99.e99;
+	DOUBLE all_avg = 0;
+	DOUBLE all_stddev = 0;
+	DOUBLE all_minval = 99.e99;
+	DOUBLE all_maxval = -99.e99;
 	init_progress_bar(Nimg);
 	int barstep = XMIPP_MAX(1, Nimg / 120);
 	for (long int i = 0; i < Nimg; i++)
@@ -730,8 +733,8 @@ void Preprocessing::runOperateOnInputFile(FileName fn_operate_on)
 }
 
 
-void Preprocessing::performPerImageOperations(Image<double> &Ipart, FileName fn_output_img_root, int nframes, long int image_nr, long int nr_of_images,
-		double &all_avg, double &all_stddev, double &all_minval, double &all_maxval)
+void Preprocessing::performPerImageOperations(Image<DOUBLE> &Ipart, FileName fn_output_img_root, int nframes, long int image_nr, long int nr_of_images,
+		DOUBLE &all_avg, DOUBLE &all_stddev, DOUBLE &all_minval, DOUBLE &all_maxval)
 {
 
 	Ipart().setXmippOrigin();
@@ -740,16 +743,16 @@ void Preprocessing::performPerImageOperations(Image<double> &Ipart, FileName fn_
 
 	if (do_rewindow) rewindow(Ipart, window);
 
-	if (do_normalise) normalise(Ipart, bg_radius, white_dust_stddev, black_dust_stddev);
+	if (do_normalise) normalise(Ipart, bg_radius, white_dust_stddev, black_dust_stddev, do_ramp);
 
 	if (do_invert_contrast) invert_contrast(Ipart);
 
 	// For movies: multiple the image intensities by sqrt(nframes) so the stddev in the average of the normalised frames is again 1
 	if (nframes > 1)
-		Ipart() *= sqrt((double)nframes/(double)avg_n_frames);
+		Ipart() *= sqrt((DOUBLE)nframes/(DOUBLE)avg_n_frames);
 
 	// Calculate mean, stddev, min and max
-	double avg, stddev, minval, maxval;
+	DOUBLE avg, stddev, minval, maxval;
 	Ipart().computeStats(avg, stddev, minval, maxval);
 
 	if (Ipart().getDim() == 3)
